fEvaluateG <- function(mYtilde, mR, mAopt, mBopt, iD, iT){
  
  mG = matrix(0,iD,iT); 
  for(t in (1:iT)){ 
    for(i in (1:iD)){
      vAopt  = mAopt[[i]];
      vBopt  = mBopt[[i]];
      
      tmp <-  matrix(0,length(vAopt),1);
      for(k in (1:length(vAopt))){
        tmp[k] <- vAopt[k] * (mR[i,] %*% mYtilde[,t]) -  vBopt[k];  
      }
      
      mG[i,t] <- min(tmp); 
    }
  }
  
  dG = iT^-1 * sum(mG); 
  
  return(dG); 
}

fEvaluateGder <- function(mYtilde, mR, mAopt, mBopt, iD, iT, mC){
  
  mK     = matrix(0,iD,iT); 
  mBfit  = matrix(0,iD, iT);
  for(t in (1:iT)){ 
    for(i in (1:iD)){
      
      vAopt  = mAopt[[i]];
      vBopt  = mBopt[[i]];
      
      tmp <-  matrix(0,length(vAopt),1);
      for(k in (1:length(vAopt))){
        tmp[k] <- vAopt[k] * (mR[i,] %*% mYtilde[,t]) -  vBopt[k];  
      }
      
      indexK  <- which.min(tmp); 
      mK[i,t] <- indexK;  
      
      mBfit[i,t] <- vAopt[mK[i,t]];
    }
  }
  
  
  dGrad <- iT^-1 *  sum( mBfit * (mC %*% mYtilde) ) ; 
  
  return(dGrad); 
}



fNP_ICA  <- function(mY, iD, iT, mRstart){
  # pre-whitening of data 
  mSigmaHat <- iT^-1 * (mY %*% t(mY));
  mLigmaHat <- t(chol(mSigmaHat)); 
  mYtilde   <- inv(mLigmaHat) %*% mY; 
  
  mR     <- mRstart;  # initialize rotation matrix  
  Yplus  <- cbind(c(0,-2^(-0.5)),c(2^(-0.5),0));   # basis for skew-symmetric matrices
  Yminus <- -Yplus; 
  
  dGold  <- -10;      # initialize log likelihood value 
  for(s in (1:100)) {
    
    # UPDATE LOG CONCAVE DENSITIES given R 
    mLogLik <- matrix(0,iD,iT); 
    mAopt   <- c();
    mBopt   <- c(); 
    
    for(k in (1:iD)){
      lcd <- mlelcd( t(mR[k,] %*% mYtilde) );
      mAopt[[k]]  <- as.matrix(lcd$b);
      mBopt[[k]]  <- as.matrix(lcd$beta);
      mLogLik[k,] <- as.matrix(lcd$logMLE);
    }
    
    # UPDATE R given log concave densities 
    dGradPlus  <- fEvaluateGder(mYtilde, mR, mAopt, mBopt, iD, iT, mR %*% Yplus);
    dGradminus <- fEvaluateGder(mYtilde, mR, mAopt, mBopt, iD, iT, mR %*% Yminus);
    
    if(dGradPlus > dGradminus){
      Ymax <- Yplus;
    } else {
      Ymax <- Yminus;
    }
    
    dGw    <- fEvaluateG(mYtilde, mR, mAopt, mBopt, iD, iT);
    dGderw <- fEvaluateGder(mYtilde, mR, mAopt, mBopt, iD, iT, mR * Ymax);
    
    alpha  <- 0.3;
    lambda <- 0.5;
    eps    <- 1;
    
    
    while( eps > 10^-7 ){
      
      mRnew <- mR %*% expm( drop(eps) * Ymax );
      dGnew <- fEvaluateG(mYtilde, mRnew, mAopt, mBopt, iD, iT);
      
      dRHS  <- dGw + alpha * eps *  dGderw;
      
      if(dGnew > dRHS){
        mR <- mRnew;
        break;
      }else{
        eps <- eps * lambda;
      }
    }
    
    # CHECK convergence 
    if( ((dGnew - dGold) / abs(dGold)) < 10^-7 ){
      break;
    }else{
      dGold <- dGnew;
    }
    
    iCount <- s;
  }
  
  mAinvHat   <- mLigmaHat * inv(mR);
  mAhat      <- inv(mAinvHat);
  
  return(mAhat); 
}