function mAderl =  fDerAmatrix(vA, iK, l)

    % derivative of the Cayley transform 
    % A = (I - O) * (I + O)^-1 wrt to an element of O 
    % we have 
    % dA/dO = - (dO/dO_l) (I+O)^-1 - (I - O) *   (I + O)^-1 * (dO /dO_l) * (I + O)^-1
    % dO/dO_l depends on the size iK and l chosen 
    
    % make skew-symmetric matrix mO
    mO = zeros(iK,iK); 
    s  = 1;
    for k=1:iK
       mO(k,k+1:end) = vA(s:s+iK-k-1);
           
       mO(k+1:end,k) = -vA(s:s+iK-k-1); 
           
       s = s + size(vA(s:s+iK-k-1),1);
    end
    
    % make derivative of skew-symmetric matrix mO
    vL    = zeros(size(vA,1) ,1);
    vL(l) = 1; 
    mOdl  = zeros(iK,iK);
    s  = 1;
    for k=1:iK
       mOdl(k,k+1:end) = vL(s:s+iK-k-1);
           
       mOdl(k+1:end,k) = -vL(s:s+iK-k-1); 
           
       s = s + size(vL(s:s+iK-k-1),1);
    end
 
    % make derivative mA 
    mAderl = -mOdl * inv(eye(iK) + mO) - (eye(iK) - mO) * inv(eye(iK) + mO) * mOdl * inv(eye(iK) + mO);

end

%     detailed without loops 
%     if iK == 2
%        mO = [0 -vA(1) ; vA(1) 0 ]; 
%        
%        vL    = zeros(size(vA,1) ,1);
%        vL(l) = 1; 
%        mOdl = [0 -vL(1) ; vL(1) 0 ]; 
%     end
%     if iK == 3
%        mO = [0 vA(1) vA(2) ; -vA(1) 0 vA(3) ; -vA(2) -vA(3) 0 ]; 
%        
%        vL    = zeros(size(vA,1) ,1);
%        vL(l) = 1;
%        
%        mOdl  = [0 vL(1) vL(2) ; -vL(1) 0 vL(3) ; -vL(2) -vL(3) 0 ]; 
%     end
% 
%     if iK == 5
%        mO = [0 vA(1) vA(2) vA(3) vA(4) ; -vA(1) 0 vA(5) vA(6) vA(7) ; -vA(2) -vA(5) 0 vA(8) vA(9) ; -vA(3) -vA(6) -vA(8) 0 vA(10) ; -vA(4) -vA(7) -vA(9) -vA(10) 0 ]; 
%     
%        vL    = zeros(size(vA,1) ,1);
%        vL(l) = 1;
%        
%        mOdl = [0 vL(1) vL(2) vL(3) vL(4) ; -vL(1) 0 vL(5) vL(6) vL(7) ; -vL(2) -vL(5) 0 vL(8) vL(9) ; -vL(3) -vL(6) -vL(8) 0 vL(10) ; -vL(4) -vL(7) -vL(9) -vL(10) 0 ]; 
%     end


