function [mAhat, mAhatEff, dLL, dLLeff] =  fGetTensorMin(iR, iType, mY, iWhite, iStarts, vProvidedStartingValues)
   
    [iD, iT]         = size(mY); 
    iL               = iD * (iD-1) / 2; 

    if iWhite == 0    % no pre-whitening 
        
        if nargin<6   % no explicit initialization given 
  
            mSigmaInit       = iT^-1 * ( (mY - mean(mY,2)) * (mY - mean(mY,2))');         
            mLigmaInit       = inv(chol(mSigmaInit,'lower'));
            vDinit           = log(diag(mLigmaInit)); 
            mLigmaInit(1:iD+1:end) = vDinit;    

            mEstResults      = cell([iStarts,2]);
            mEstResultsEff   = cell([iStarts,2]);

            mInits           = unifrnd(-5,5,iStarts,iL);    

            for ss = 1:iStarts 
                
                init             = mInits(ss,:)';

                [vOrders,  vInd] = fGetOrders(iType, iR, iD);              % get corresponding zero indices input: [type, iR, iD]

                vStart           = [vech(mLigmaInit) ; init];              % starting values 
                mW               = eye(size(vOrders,1));                   % identity weighting matrix 

                % unweighted GMM 
                options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
                [vParHat,dLL,iExitflag] = fminunc(@fJcommonvariance,vStart,options,vOrders,vInd,iD,mW, mY);
                if iExitflag<=0; warning('Identity weight did not converge, exit flag: %g',iExitflag); end

                mLigmaHat        = unvech(vParHat(1:iD*(iD+1)/2));
                vDhat            = exp(diag(mLigmaHat)); 
                mLigmaHat(1:iD+1:end) = vDhat;    
                vAlphaHat        = vParHat(iD*(iD+1)/2 + 1:end); 
                mRhat            = fAmatrix(vAlphaHat, iD);
                mAhat            = mRhat' * mLigmaHat; 

                mEstResults{ss,1} = dLL;
                mEstResults{ss,2} = mAhat;  

                % continuous updating estimator 
                options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
                [vParHat,dLLeff,iExitflag] = fminunc(@fJcommonvariance2,vStart,options,vOrders,vInd,iD, iT, mY);
                if iExitflag<=0; warning('CUE did not converge, exit flag: %g',iExitflag); end

                mLigmaHat = unvech(vParHat(1:iD*(iD+1)/2));
                vDhat     = exp(diag(mLigmaHat)); 
                mLigmaHat(1:iD+1:end) = vDhat;    
                vAlphaHat = vParHat(iD*(iD+1)/2 + 1:end); 
                mRhat     = fAmatrix(vAlphaHat, iD);
                mAhatEff  = mRhat' * mLigmaHat; 

                mEstResultsEff{ss,1} = dLLeff;
                mEstResultsEff{ss,2} = mAhatEff;
            end


            [~ , dMinIndex] = min(cell2mat(  mEstResults(:,1) )); 
            mAhat = mEstResults{dMinIndex,2}; 
            dLL   = min(cell2mat(  mEstResults(:,1) )); 


            [~ , dMinIndex] = min(cell2mat( mEstResultsEff(:,1) )); 
            mAhatEff = mEstResultsEff{dMinIndex,2}; 
            dLLeff   = min(cell2mat( mEstResultsEff(:,1) )); 

        else

            [vOrders,  vInd] = fGetOrders(iType, iR, iD);              % get corresponding zero indices input: [type, iR, iD]

            vStart           = vProvidedStartingValues;                % starting values 
            mW               = eye(size(vOrders,1));                   % initial weighting matrix 

            % unweighted GMM 
            options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
            [vParHat,dLL,iExitflag] = fminunc(@fJcommonvariance,vStart,options,vOrders,vInd,iD,mW, mY);
            if iExitflag<=0; warning('Identity weight did not converge, exit flag: %g',iExitflag); end

            mLigmaHat        = unvech(vParHat(1:iD*(iD+1)/2));
            vDhat            = exp(diag(mLigmaHat)); 
            mLigmaHat(1:iD+1:end) = vDhat;    
            vAlphaHat        = vParHat(iD*(iD+1)/2 + 1:end); 
            mRhat            = fAmatrix(vAlphaHat, iD);
            mAhat            = mRhat' * mLigmaHat; 

            % continuous updating estimator 
            options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
            [vParHat,dLLeff,iExitflag] = fminunc(@fJcommonvariance2,vStart,options,vOrders,vInd,iD, iT, mY);
            if iExitflag<=0; warning('CUE did not converge, exit flag: %g',iExitflag); end

            mLigmaHat = unvech(vParHat(1:iD*(iD+1)/2));
            vDhat     = exp(diag(mLigmaHat)); 
            mLigmaHat(1:iD+1:end) = vDhat;    
            vAlphaHat = vParHat(iD*(iD+1)/2 + 1:end); 
            mRhat     = fAmatrix(vAlphaHat, iD);
            mAhatEff  = mRhat' * mLigmaHat; 

        end
    end   
    
    if iWhite == 1   % separately estimate Sigms^1/2 and R 
        
        mSigmaHat        = iT^-1 * (mY * mY');         
        mLigmaHat        = chol(mSigmaHat,'lower');
        mYtilde          = mLigmaHat * mY; 
        
        
        if nargin<6   % no explicit initialization given 
 
            mEstResults      = cell([iStarts,2]);
            mEstResultsEff   = cell([iStarts,2]);

            mInits           = unifrnd(-5,5,iStarts,iL);    

            for ss = 1:iStarts 

                init             = mInits(ss,:)';


                [vOrders,  vInd] = fGetOrders(iType, iR, iD);              % get corresponding zero indices input: [type, iR, iD]

                vOrders          = vOrders(iD*(iD+1)/2+1:end,:);           % drop indices that correspond to Sigma^1/2 estimation      
                vInd             = vInd(iD*(iD+1)/2+1:end,:);
                
                vStart           = init;                                   % starting values only for rotation matrix 
                mW               = eye(size(vOrders,1));                   % identity weighting matrix 

                options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
                [vParHat,dLL,iExitflag] = fminunc(@fJcommonvarianceWhite,vStart,options,vOrders,vInd,iD,mW, mYtilde);
                if iExitflag<=0; warning('Identity weight did not converge, exit flag: %g',iExitflag); end

                mRhat            = fAmatrix(vParHat, iD);
                mAhat            = mRhat' * inv(mLigmaHat); 

                mEstResults{ss,1} = dLL;
                mEstResults{ss,2} = mAhat;  

                options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
                [vParHat,dLLeff,iExitflag] = fminunc(@fJcommonvariance2White,vStart,options,vOrders,vInd,iD, iT, mYtilde);
                if iExitflag<=0; warning('CUE did not converge, exit flag: %g',iExitflag); end

                mRhat     = fAmatrix(vParHat, iD);
                mAhatEff  = mRhat' * inv(mLigmaHat); 

                mEstResultsEff{ss,1} = dLLeff;
                mEstResultsEff{ss,2} = mAhatEff;
            end


            [~ , dMinIndex] = min(cell2mat(  mEstResults(:,1) )); 
            mAhat = mEstResults{dMinIndex,2}; 
            dLL   = min(cell2mat(  mEstResults(:,1) )); 


            [~ , dMinIndex] = min(cell2mat( mEstResultsEff(:,1) )); 
            mAhatEff = mEstResultsEff{dMinIndex,2}; 
            dLLeff   = min(cell2mat( mEstResultsEff(:,1) )); 

        else

            [vOrders,  vInd] = fGetOrders(iType, iR, iD);              % get corresponding zero indices input: [type, iR, iD]

            vStart           = vProvidedStartingValues;                % starting values 
            mW               = eye(size(vOrders,1));                   % initial weighting matrix 

            options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
            [vParHat,dLL,iExitflag] = fminunc(@fJcommonvarianceWhite,vStart,options,vOrders,vInd,iD,mW, mY);
            if iExitflag<=0; warning('Identity weight did not converge, exit flag: %g',iExitflag); end
            
            mRhat            = fAmatrix(vParHat, iD);
            mAhat            = mRhat' * mLigmaHat; 

            options          = optimoptions('fminunc','Display','notify','MaxFunctionEvaluations',1e4, 'MaxIterations',1e3);
            [vParHat,dLLeff,iExitflag] = fminunc(@fJcommonvariance2White,vStart,options,vOrders,vInd,iD, iT, mY);
            if iExitflag<=0; warning('CUE did not converge, exit flag: %g',iExitflag); end

            mRhat     = fAmatrix(vParHat, iD);
            mAhatEff  = mRhat' * mLigmaHat; 
        
        
        end
        
    end
   
    
    

end